diff --git a/Makefile.sources b/Makefile.sources
index d98c4a2..dced7d7 100644
--- a/Makefile.sources
+++ b/Makefile.sources
@@ -13,6 +13,7 @@
 
 sources=$(filter-out $(auto_gen), $(wildcard *.c) $(wildcard mem/*.c) \
 		$(wildcard parser/*.c) $(wildcard parser/digest/*.c) \
+		$(wildcard parser/sdp/*.c) \
 		$(wildcard parser/contact/*.c) $(wildcard db/*.c) \
 		$(wildcard rand/*.c) $(wildcard rand/isaac/*.c) ) \
 		$(wildcard cfg/*.c) $(auto_gen)
diff --git a/modules/tm/sip_msg.c b/modules/tm/sip_msg.c
index 90a3fa1..83f5192 100644
--- a/modules/tm/sip_msg.c
+++ b/modules/tm/sip_msg.c
@@ -522,6 +522,9 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len )
 	/* sip msg structure */
 	memcpy( new_msg , org_msg , sizeof(struct sip_msg) );
 
+	/* avoid copying pointer to un-clonned structures */
+	new_msg->sdp = NULL;
+
 	new_msg->msg_flags |= FL_SHM_CLONE;
 	p += ROUND4(sizeof(struct sip_msg));
 	new_msg->add_rm = 0;
diff --git a/parser/msg_parser.c b/parser/msg_parser.c
index 74f91f5..56e9c71 100644
--- a/parser/msg_parser.c
+++ b/parser/msg_parser.c
@@ -39,6 +39,7 @@
  *  2005-03-02  free_via_list(vb) on via parse error (andrei)
  *  2007-01-26  parser extended to support Identity, Identity-info and Date
  *		header fields (gergo)
+ *  2009-03-23  added sdp parsing support (osas)
  */
 
 
@@ -694,6 +695,7 @@ void free_sip_msg(struct sip_msg* msg)
 	if (msg->new_uri.s) { pkg_free(msg->new_uri.s); msg->new_uri.len=0; }
 	if (msg->dst_uri.s) { pkg_free(msg->dst_uri.s); msg->dst_uri.len=0; }
 	if (msg->headers)     free_hdr_field_lst(msg->headers);
+	if (msg->sdp)         free_sdp(&(msg->sdp));
 	if (msg->add_rm)      free_lump_list(msg->add_rm);
 	if (msg->body_lumps)  free_lump_list(msg->body_lumps);
 	if (msg->reply_lump)   free_reply_lump(msg->reply_lump);
diff --git a/parser/msg_parser.h b/parser/msg_parser.h
index 624c2ab..fd5f8b8 100644
--- a/parser/msg_parser.h
+++ b/parser/msg_parser.h
@@ -43,11 +43,12 @@
  *  2007-01-26  added date, identity, identity_info header fields
  *		to sip_msg (gergo)
  *  2007-03-14  added SIP_MSG_START(msg) macro
+ *  2009-03-23  added sdp parsing support (osas)
  */
 
 
-#ifndef msg_parser_h
-#define msg_parser_h
+#ifndef MSG_PARSER_H
+#define MSG_PARSER_H
 
 
 #include "../comp_defs.h"
@@ -64,6 +65,7 @@
 #include "parse_retry_after.h"
 #include "hf.h"
 #include "../error.h"
+#include "sdp/sdp.h"
 
 
 /* convenience short-cut macros */
@@ -259,6 +262,8 @@ typedef struct sip_msg {
 	struct hdr_field* ppi;
 	struct hdr_field* path;
 
+	struct sdp_info* sdp;
+
 	char* eoh;        /* pointer to the end of header (if found) or null */
 	char* unparsed;   /* here we stopped parsing*/
 
